import _isArray from "lodash/isArray";
import _isString from "lodash/isString";
import _omit from "lodash/omit";

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React from 'react';
import { StyleSheet } from 'react-native';
import View from 'react-native-web-ui-components/View';
import Row from 'react-native-web-ui-components/Row';
import Column from 'react-native-web-ui-components/Column';
import Screen from 'react-native-web-ui-components/Screen';
import { Helmet, style } from 'react-native-web-ui-components/Helmet';
import { getComponent, withPrefix } from '../../utils';
/* eslint react/prop-types: 0 */

const styles = StyleSheet.create({
  labelTop: {
    fontWeight: 'bold',
    paddingBottom: 5
  },
  label: {
    fontWeight: 'bold',
    paddingTop: 10,
    paddingBottom: 5
  },
  grid: {
    marginLeft: -10,
    alignItems: 'flex-start'
  },
  item: {
    paddingLeft: 10
  }
});
const attributes = ['type', 'children', 'style', 'columns'];

const getMeta = schema => {
  if (schema.type === 'array') {
    return [];
  }

  return {};
};

const createProperty = (property, gridItem, index, params) => {
  const {
    name,
    schema,
    fields
  } = params;
  const propertySchema = schema.properties[property];
  const propertyName = withPrefix(property, name);

  if (!propertySchema) {
    const UnexistentProperty = () => null;

    UnexistentProperty.key = propertyName;
    return UnexistentProperty;
  }

  const PropertyComponent = getComponent(propertySchema.type, 'Field', fields);

  if (!PropertyComponent) {
    const UnexistentPropertyComponent = () => null;

    UnexistentPropertyComponent.key = propertyName;
    return UnexistentPropertyComponent;
  }

  let PropertyContainer;
  let propertyContainerProps;

  if (gridItem.type === 'grid') {
    const columns = gridItem.columns || [];
    const column = (_isArray(columns) ? columns[index] : columns) || {};
    PropertyContainer = Row;
    propertyContainerProps = {
      style: [Screen.getType() !== 'xs' ? styles.item : null, {
        zIndex: gridItem.children.length - index
      }, column.style || null],
      ...column
    };
  } else {
    PropertyContainer = React.Fragment;
    propertyContainerProps = {};
  }

  const Property = ({
    value,
    meta,
    errors,
    uiSchema,
    ...props
  }) => React.createElement(PropertyContainer, _extends({
    key: propertyName
  }, propertyContainerProps), React.createElement(PropertyComponent, _extends({}, props, {
    value: value && value[property],
    meta: meta && meta[property] || getMeta(propertySchema),
    errors: errors && errors[property],
    name: propertyName,
    schema: propertySchema,
    uiSchema: uiSchema[property],
    gridItemType: gridItem.type,
    gridItemIndex: index,
    gridItemLength: gridItem.children.length,
    zIndex: gridItem.children.length - index
  })));

  Property.key = propertyName;
  return Property;
};

const createGridItem = (gridItem, key, zIndex, first, params) => {
  const {
    widgets
  } = params;

  if (gridItem.type === 'label') {
    const Widget = widgets.LabelWidget;

    const Label = props => React.createElement(Widget, _extends({}, props, {
      key: key,
      hasError: false,
      hasTitle: true,
      toggleable: false,
      style: [first ? styles.labelTop : styles.label, gridItem.style]
    }), gridItem.children);

    Label.key = key;
    return Label;
  }

  let Wrapper;

  if (gridItem.type === 'column') {
    Wrapper = Column;
  } else if (gridItem.type === 'view') {
    Wrapper = View;
  } else {
    Wrapper = Row;
  }

  const gridStyle = Screen.getType() !== 'xs' && gridItem.type === 'grid' ? styles.grid : null;
  const items = gridItem.children.map((child, i) => {
    if (_isString(child)) {
      return createProperty(child, gridItem, i, params);
    }

    return createGridItem(child, `${key}-${i}`, gridItem.children.length - i, i === 0, params);
  });

  const GridItem = props => React.createElement(Wrapper, _extends({
    className: `FormGridItem__${gridItem.type}`
  }, _omit(gridItem, attributes), {
    style: [gridItem.style, {
      zIndex
    }, gridStyle]
  }), items.map(Child => React.createElement(Child, _extends({
    key: Child.key
  }, props))));

  GridItem.key = key;
  return GridItem;
};

const createGrid = (grid, params) => {
  const items = grid.map((gridItem, i) => createGridItem(gridItem, `${params.name}-${i}`, grid.length - i, i === 0, params));
  return props => {
    const currentStyle = props.style; // eslint-disable-line

    return React.createElement(Row, {
      style: currentStyle
    }, Screen.getType() !== 'xs' ? React.createElement(Helmet, null, React.createElement("style", null, `
                [data-class~="FormGridItem__grid"] {
                  width: calc(100% + 10px);
                }
              `)) : null, items.map(GridItem => React.createElement(GridItem, _extends({
      key: GridItem.key
    }, props))));
  };
};

export default createGrid;